@@ -1,8 +1,7 @@
-import json, math, hashlib, threading, os, subprocess, sys, time
+import json, math, hashlib, threading, os, subprocess, time
 import numbers, base64, binascii, random, requests, paramiko
 from psutil import Process
 from datetime import datetime
-from random import randrange
 from .ec_classes import FatalError, RuntimeWarning, RuntimeError, AssertionError, NoValueError, NoValueRuntimeError, Condition, Object
 from .ec_handler import Handler
 from .ec_timestamp import getTimestamp
@@ -18,6 +17,26 @@ class Core(Handler):
     
     def noSymbolWarning(self):
         self.warning(f'Symbol "{self.getToken()}" not found')
+    
+    def processOr(self, command, orHere):
+        self.add(command)
+        if self.peek() == 'or':
+            self.nextToken()
+            self.nextToken()
+            # Add a 'goto' to skip the 'or'
+            cmd = {}
+            cmd['lino'] = command['lino']
+            cmd['domain'] = 'core'
+            cmd['keyword'] = 'gotoPC'
+            cmd['goto'] = 0
+            cmd['debug'] = False
+            skip = self.getPC()
+            self.add(cmd)
+            # Process the 'or'
+            self.getCommandAt(orHere)['or'] = self.getPC()
+            self.compileOne()
+            # Fixup the skip
+            self.getCommandAt(skip)['goto'] = self.getPC()
 
     #############################################################################
     # Keyword handlers
@@ -454,24 +473,7 @@ class Core(Handler):
                         timeout['type'] = 'int'
                         timeout['content'] = 5
                         command['timeout'] = timeout
-                    self.add(command)
-                    if self.peek() == 'or':
-                        self.nextToken()
-                        self.nextToken()
-                        # Add a 'goto' to skip the 'or'
-                        cmd = {}
-                        cmd['lino'] = command['lino']
-                        cmd['domain'] = 'core'
-                        cmd['keyword'] = 'gotoPC'
-                        cmd['goto'] = 0
-                        cmd['debug'] = False
-                        skip = self.getPC()
-                        self.add(cmd)
-                        # Process the 'or'
-                        self.getCommandAt(get)['or'] = self.getPC()
-                        self.compileOne()
-                        # Fixup the skip
-                        self.getCommandAt(skip)['goto'] = self.getPC()
+                    self.processOr(command, get)
                     return True
         return False
 
@@ -591,39 +593,29 @@ class Core(Handler):
             self.program.pc += 1
         return self.program.pc
 
+    # Import one or more variables
     def k_import(self, command):
-        if self.peek() == 'plugin':
-            # Import a plugin
+        imports = []
+        while True:
+            keyword = self.nextToken()
+            name = self.nextToken()
+            item = [keyword, name]
+            imports.append(item)
+            self.symbols[name] = self.getPC()
+            variable = {}
+            variable['domain'] = None
+            variable['name'] = name
+            variable['keyword'] = keyword
+            variable['import'] = None
+            variable['used'] = False
+            variable['hasValue'] = True if keyword == 'variable' else False
+            self.add(variable)
+            if self.peek() != 'and':
+                break
             self.nextToken()
-            clazz = self.nextToken()
-            if self.nextIs('from'):
-                source = self.nextToken()
-                self.program.importPlugin(f'{source}:{clazz}')
-                return True
-            return False
-        else:
-            # Import one or more variables
-            imports = []
-            while True:
-                keyword = self.nextToken()
-                name = self.nextToken()
-                item = [keyword, name]
-                imports.append(item)
-                self.symbols[name] = self.getPC()
-                variable = {}
-                variable['domain'] = None
-                variable['name'] = name
-                variable['keyword'] = keyword
-                variable['import'] = None
-                variable['used'] = False
-                variable['hasValue'] = True if keyword == 'variable' else False
-                self.add(variable)
-                if self.peek() != 'and':
-                    break
-                self.nextToken()
-            command['imports'] = json.dumps(imports)
-            self.add(command)
-            return True
+        command['imports'] = json.dumps(imports)
+        self.add(command)
+        return True
 
     def r_import(self, command):
         exports = self.program.exports
@@ -752,24 +744,7 @@ class Core(Handler):
                         command['file'] = self.getValue()
                     command['or'] = None
                     load = self.getPC()
-                    self.add(command)
-                    if self.peek() == 'or':
-                        self.nextToken()
-                        self.nextToken()
-                        # Add a 'goto' to skip the 'or'
-                        cmd = {}
-                        cmd['lino'] = command['lino']
-                        cmd['domain'] = 'core'
-                        cmd['keyword'] = 'gotoPC'
-                        cmd['goto'] = 0
-                        cmd['debug'] = False
-                        skip = self.getPC()
-                        self.add(cmd)
-                        # Process the 'or'
-                        self.getCommandAt(load)['or'] = self.getPC()
-                        self.compileOne()
-                        # Fixup the skip
-                        self.getCommandAt(skip)['goto'] = self.getPC()
+                    self.processOr(command, load)
                     return True
         else:
             FatalError(self.compiler, f'I don\'t understand \'{self.getToken()}\'')
@@ -786,6 +761,11 @@ class Core(Handler):
                 with sftp.open(path, 'r') as remote_file: content = remote_file.read().decode()
             except:
                 errorReason = f'Unable to read from {path}'
+                if command['or'] != None:
+                    print(f'Exception "{errorReason}": Running the "or" clause')
+                    return command['or']
+                else:
+                    RuntimeError(self.program, f'Error: {errorReason}')
         else:
             filename = self.getRuntimeValue(command['file'])
             try:
@@ -1024,24 +1004,7 @@ class Core(Handler):
             command['result'] = None
         command['or'] = None
         post = self.getPC()
-        self.add(command)
-        if self.peek() == 'or':
-            self.nextToken()
-            self.nextToken()
-            # Add a 'goto' to skip the 'or'
-            cmd = {}
-            cmd['lino'] = command['lino']
-            cmd['domain'] = 'core'
-            cmd['keyword'] = 'gotoPC'
-            cmd['goto'] = 0
-            cmd['debug'] = False
-            skip = self.getPC()
-            self.add(cmd)
-            # Process the 'or'
-            self.getCommandAt(post)['or'] = self.getPC()
-            self.compileOne()
-            # Fixup the skip
-            self.getCommandAt(skip)['goto'] = self.getPC()
+        self.processOr(command, post)
         return True
 
     def r_post(self, command):
@@ -1138,7 +1101,8 @@ class Core(Handler):
                     if 'hasValue' in symbolRecord and symbolRecord['hasValue'] == False:
                         FatalError(self.compiler, f'Symbol {symbolRecord["name"]} is not a value holder')
                     else:
-                        self.add(command)
+                        command['or'] = None
+                        self.processOr(command, self.getPC())
                         return True
                 else:
                     FatalError(self.compiler, f'Symbol {self.getToken()} is not a variable')
@@ -1147,7 +1111,10 @@ class Core(Handler):
     def r_put(self, command):
         value = self.evaluate(command['value'])
         if value == None:
-            return -1
+            if command['or'] != None:
+                return command['or']
+            else:
+                RuntimeError(self.program, f'Error: could not compute value')
         symbolRecord = self.getVariable(command['target'])
         if not symbolRecord['hasValue']:
             NoValueRuntimeError(self.program, symbolRecord)
@@ -1304,24 +1271,7 @@ class Core(Handler):
             command['file'] = self.getValue()
         command['or'] = None
         save = self.getPC()
-        self.add(command)
-        if self.peek() == 'or':
-            self.nextToken()
-            self.nextToken()
-            # Add a 'goto' to skip the 'or'
-            cmd = {}
-            cmd['lino'] = command['lino']
-            cmd['domain'] = 'core'
-            cmd['keyword'] = 'gotoPC'
-            cmd['goto'] = 0
-            cmd['debug'] = False
-            skip = self.getPC()
-            self.add(cmd)
-            # Process the 'or'
-            self.getCommandAt(save)['or'] = self.getPC()
-            self.compileOne()
-            # Fixup the skip
-            self.getCommandAt(skip)['goto'] = self.getPC()
+        self.processOr(command, save)
         return True
 
     def r_save(self, command):
@@ -1336,6 +1286,11 @@ class Core(Handler):
                 with sftp.open(path, 'w') as remote_file: remote_file.write(content)
             except:
                 errorReason = 'Unable to write to {path}'
+                if command['or'] != None:
+                    print(f'Exception "{errorReason}": Running the "or" clause')
+                    return command['or']
+                else:
+                    RuntimeError(self.program, f'Error: {errorReason}')
         else:
             filename = self.getRuntimeValue(command['file'])
             if filename.endswith('.json'): content = json.dumps(content)
@@ -1798,13 +1753,21 @@ class Core(Handler):
 
     # Use a plugin module
     def k_use(self, command):
-        if self.nextIs('plugin'):
-            if self.nextIs('graphics'):
-                print('Loading graphics module')
-                from .ec_pyside import Graphics
-                self.program.graphics = Graphics
-                self.program.useClass(Graphics)
+        if self.peek() == 'plugin':
+            # Import a plugin
+            self.nextToken()
+            clazz = self.nextToken()
+            if self.nextIs('from'):
+                source = self.nextToken()
+                self.program.importPlugin(f'{source}:{clazz}')
                 return True
+            return False
+        elif self.nextIs('graphics'):
+            print('Loading graphics module')
+            from .ec_pyside import Graphics
+            self.program.graphics = Graphics
+            self.program.useClass(Graphics)
+            return True
         return False
 
     # Declare a general-purpose variable
@@ -2395,7 +2358,7 @@ class Core(Handler):
         try:
             value['content'] = json.loads(item)
         except:
-            RuntimeError(self.program, 'Cannot encode value')
+            value = None
         return value
 
     def v_keys(self, v):
@@ -2511,7 +2474,7 @@ class Core(Handler):
         limit = self.getRuntimeValue(v['content'])
         value = {}
         value['type'] = 'int'
-        value['content'] = randrange(0, limit)
+        value['content'] = random.randrange(0, limit)
         return value
 
     def v_right(self, v):
