@@ -1,4 +1,3 @@
-
 import os
 from .ec_handler import Handler
 from PySide6.QtWidgets import (
@@ -14,7 +13,7 @@ from PySide6.QtWidgets import (
     QSizePolicy,
     QGraphicsDropShadowEffect
 )
-from PySide6.QtGui import QFont, QIcon, QPixmap, QPainter, QColor
+from PySide6.QtGui import QFont, QIcon, QPixmap, QPainter
 from PySide6.QtCore import Qt, QTimer, Signal, QRect
 
 class Keyboard(Handler):
@@ -65,12 +64,12 @@ class Keyboard(Handler):
             self._drag_active = False
             super().mouseReleaseEvent(event)
     
-    def __init__(self, program, keyboardType, receiver, caller = None, parent=None):
+    def __init__(self, program, keyboardType, receiverLayout, receivers, caller = None, parent=None):
         super().__init__(program.compiler)
 
         self.program = program
         self.keyboardType = keyboardType
-        self.receiver = receiver
+        self.receivers = receivers
 
         dialog = QDialog(caller)
         self.dialog = dialog
@@ -78,9 +77,8 @@ class Keyboard(Handler):
 #        dialog.setWindowTitle('')
         dialog.setWindowFlags(Qt.FramelessWindowHint)
         dialog.setModal(True)
-        dialog.setFixedSize(500, 250)
+        dialog.setFixedWidth(500)
         dialog.setStyleSheet('background-color: white;border:1px solid black;')
-        self.originalText = receiver.getContent()
 
         # Add drop shadow
         shadow = QGraphicsDropShadowEffect(dialog)
@@ -95,7 +93,18 @@ class Keyboard(Handler):
         border = self.Border()
         border.iconClicked.connect(self.reject)
         layout.addWidget(border)
-        layout.addWidget(VirtualKeyboard(keyboardType, 42, receiver, dialog.accept))
+        layout.addLayout(receiverLayout)
+        self.vk = VirtualKeyboard(keyboardType, 42, receivers[0], dialog.accept)
+        layout.addWidget(self.vk)
+
+        restore = []
+        index = 0
+        for receiver in receivers:
+            receiver.field.setContainer(self)
+            receiver.index = index
+            restore.append(receiver.getContent())
+            index += 1
+        self.restore = restore
         
         # Position at bottom of parent window
         dialog.show()  # Ensure geometry is calculated
@@ -107,8 +116,20 @@ class Keyboard(Handler):
 
         dialog.exec()
 
+    def setClickSource(self, field):
+        receivers = self.receivers
+        for receiver in receivers:
+            if receiver.field == field:
+                self.vk.setReceiver(receiver)
+                return
+
     def reject(self):
-        self.receiver.setContent(self.originalText)
+        selectedReceiver = self.vk.getReceiver()
+        receivers = self.receivers
+        index = 0
+        for receiver in receivers:
+            receiver.setContent(self.restore[index])
+            index += 1      
         self.dialog.reject()
 
 class TextReceiver():
@@ -118,14 +139,14 @@ class TextReceiver():
     def addCharacter(self, char):
         char = char.replace('&&', '&')
         if len(char) == 1:
-            self.field.setText(self.field.text() + char)
+            self.setContent(self.getContent() + char)
         else:
             raise ValueError("Only single characters are allowed.")
 
     def backspace(self):
-        current_text = self.field.text()
+        current_text = self.getContent()
         if current_text:
-            self.field.setText(current_text[:-1])
+            self.setContent(current_text[:-1])
 
     def setContent(self, text):
         if isinstance(self.field, QLineEdit):
@@ -184,11 +205,11 @@ class KeyboardView(QVBoxLayout):
             self.addLayout(row)
 
 class VirtualKeyboard(QStackedWidget):
-    def __init__(self, keyboardType, buttonHeight, textField, onFinished):
+    def __init__(self, keyboardType, buttonHeight, receiver, onFinished):
         super().__init__()
         self.keyboardType = keyboardType
         self.buttonHeight = buttonHeight
-        self.textField = textField
+        self.receiver = receiver
         self.onFinished = onFinished
         self.setStyleSheet('background-color: #ccc;border:none;')
 
@@ -409,11 +430,17 @@ class VirtualKeyboard(QStackedWidget):
         container = QWidget()
         container.setLayout(keyboardView)
         self.addWidget(container)
+    
+    def setReceiver(self, receiver):
+        self.receiver = receiver
+    
+    def getReceiver(self):
+        return self.receiver
 
     # Callback functions
     def onClickChar(self,keycode):
         # print(f"Key pressed: {keycode}")
-        self.textField.addCharacter(keycode)
+        self.receiver.addCharacter(keycode)
 
     def onClickShift(self,keycode):
         # print("Shift pressed")
@@ -436,13 +463,13 @@ class VirtualKeyboard(QStackedWidget):
 
     def onClickBack(self,keycode):
         # print("Backspace pressed")
-        self.textField.backspace()
+        self.receiver.backspace()
 
     def onClickSpace(self,keycode):
         # print("Space pressed")
-        self.textField.addCharacter(' ')
+        self.receiver.addCharacter(' ')
 
     def onClickEnter(self,keycode):
         # print("Enter pressed")
-        if self.keyboardType == 'multiline': self.textField.addCharacter('\n')
+        if self.keyboardType == 'multiline': self.receiver.addCharacter('\n')
         else: self.onFinished()
\ No newline at end of file
