@@ -48,7 +48,7 @@ class Program:
 		self.externalControl = False
 		self.ticker = 0
 		self.running = True
-		self.start()
+#		self.start()
 
 	# This is called at 10msec intervals by the GUI code
 	def flushCB(self):
@@ -390,7 +390,7 @@ class Program:
 # This is the program launcher
 def Main():
 	if (len(sys.argv) > 1):
-		Program(sys.argv[1])
+		Program(sys.argv[1]).start()
 	else:
 		Program('-v')
 
