@@ -1,8 +1,7 @@
 import sys
 from .ec_handler import Handler
 from .ec_classes import RuntimeError
-from .ec_keyboard import Keyboard, TextReceiver
-from PySide6.QtCore import Qt, QTimer
+from PySide6.QtCore import Qt, QTimer, Signal
 from PySide6.QtGui import QPixmap
 from PySide6.QtWidgets import (
     QApplication,
@@ -55,7 +54,7 @@ class Graphics(Handler):
     def isWidget(self, keyword):
         return keyword in [
             'layout',
-            'groupbox',
+            'group',
             'label',
             'pushbutton',
             'checkbox',
@@ -80,6 +79,34 @@ class Graphics(Handler):
         def afterShown(self):
             if 'action' in self.record: self.record['action']()
 
+    class ClickableLineEdit(QLineEdit):
+        clicked = Signal()
+
+        def __init__(self):
+            super().__init__()
+        
+        def setContainer(self, container):
+            self.container = container
+
+        def mousePressEvent(self, event):
+            self.clicked.emit()
+            super().mousePressEvent(event)
+            if self.container != None: self.container.setClickSource(self)
+
+    class ClickablePlainTextEdit(QPlainTextEdit):
+        clicked = Signal()
+
+        def __init__(self):
+            super().__init__()
+        
+        def setContainer(self, container):
+            self.container = container
+
+        def mousePressEvent(self, event):
+            self.clicked.emit()
+            super().mousePressEvent(event)
+            if self.container != None: self.container.setClickSource(self)
+
     #############################################################################
     # Keyword handlers
 
@@ -88,11 +115,12 @@ class Graphics(Handler):
     # (3) add stretch {widget} to {layout}
     # (4) add stretch to {layout}
     # (5) add spacer {size} to {layout}
+    # (6) add {widget} at {col} {row} in {grid layout}
     def k_add(self, command):
         def addToLayout():
             if self.nextIsSymbol():
                 record = self.getSymbolRecord()
-                if record['keyword'] in ['layout', 'groupbox', 'element']:
+                if record['keyword'] in ['layout', 'group', 'element']:
                     command['layout'] = record['name']
                     self.add(command)
                     return True
@@ -128,12 +156,20 @@ class Graphics(Handler):
             record = self.getSymbolRecord()
             if record['extra'] == 'gui':
                 if self.isWidget(record['keyword']):
+                    command['widget'] = record['name']
                     if self.peek() == 'to':
                         # (2)
                         record = self.getSymbolRecord()
-                        command['widget'] = record['name']
                         self.nextToken()
                         return addToLayout()
+                    elif self.peek() == 'at':
+                        # (6)
+                        self.nextToken()
+                        command['row'] = self.nextValue()
+                        command['col'] = self.nextValue()
+                        self.skip('in')
+                        return addToLayout()
+
                 else: return False
         # (1)
         value = self.getValue()
@@ -151,8 +187,21 @@ class Graphics(Handler):
         if 'value' in command:
             value = self.getRuntimeValue(command['value'])
             widget = self.getVariable(command['widget'])
-            if widget['keyword'] in ['listbox', 'combobox']:
+            if widget['keyword'] == 'listbox':
                 widget['widget'].addItem(value)
+            elif widget['keyword'] == 'combobox':
+                if isinstance(value, list): widget['widget'].addItems(value)
+                else: widget['widget'].addItem(value)
+        elif 'row' in command and 'col' in command:
+            layout = self.getVariable(command['layout'])['widget']
+            widgetVar = self.getVariable(command['widget'])
+            widget = widgetVar['widget']
+            row = self.getRuntimeValue(command['row'])
+            col = self.getRuntimeValue(command['col'])
+            if widgetVar['keyword'] == 'layout':
+                layout.addLayout(widget, row, col)
+            else:
+                layout.addWidget(widget, row, col)
         else:
             layoutRecord = self.getVariable(command['layout'])
             widget = command['widget']
@@ -167,11 +216,11 @@ class Graphics(Handler):
                 layout = layoutRecord['widget']
                 stretch = 'stretch' in command
                 if widgetRecord['keyword'] == 'layout':
-                    if layoutRecord['keyword'] == 'groupbox':
+                    if layoutRecord['keyword'] == 'group':
                         if widgetRecord['keyword'] == 'layout':
                             layout.setLayout(widget)
                         else:
-                            RuntimeError(self.program, 'Can only add a layout to a groupbox')
+                            RuntimeError(self.program, 'Can only add a layout to a group')
                     else:
                         if stretch: layout.addLayout(widget, stretch=1)
                         else: layout.addLayout(widget)
@@ -297,6 +346,7 @@ class Graphics(Handler):
 
     def k_createLabel(self, command):
         text = self.compileConstant('')
+        size = self.compileConstant(20)
         while True:
             token = self.peek()
             if token == 'text':
@@ -304,7 +354,7 @@ class Graphics(Handler):
                 text = self.nextValue()
             elif token == 'size':
                 self.nextToken()
-                command['size'] = self.nextValue()
+                size = self.nextValue()
             elif token == 'align':
                 self.nextToken()
                 token = self.nextToken()
@@ -312,6 +362,7 @@ class Graphics(Handler):
                     command['align'] = token
             else: break
         command['text'] = text
+        command['size'] = size
         self.add(command)
         return True
 
@@ -334,17 +385,25 @@ class Graphics(Handler):
         if self.peek() == 'text':
             self.nextToken()
             text = self.nextValue()
-        else: text = ''
+        else: text = self.compileConstant('')
         command['text'] = text
         self.add(command)
         return True
 
     def k_createLineEdit(self, command):
-        if self.peek() == 'size':
-            self.nextToken()
-            size = self.nextValue()
-        else: size = self.compileConstant(10)
+        text = self.compileConstant('')
+        size = self.compileConstant(40)
+        while True:
+            token = self.peek()
+            if token == 'text':
+                self.nextToken()
+                text = self.nextValue()
+            elif token == 'size':
+                self.nextToken()
+                size = self.nextValue()
+            else: break;
         command['size'] = size
+        command['text'] = text
         self.add(command)
         return True
 
@@ -434,7 +493,7 @@ class Graphics(Handler):
             keyword = record['keyword']
             if keyword == 'window': return self.k_createWindow(command)
             elif keyword == 'layout': return self.k_createLayout(command)
-            elif keyword == 'groupbox': return self.k_createGroupBox(command)
+            elif keyword == 'group': return self.k_createGroupBox(command)
             elif keyword == 'label': return self.k_createLabel(command)
             elif keyword == 'pushbutton': return self.k_createPushbutton(command)
             elif keyword == 'checkbox': return self.k_createCheckBox(command)
@@ -462,23 +521,27 @@ class Graphics(Handler):
         return self.nextPC()
     
     def r_createLayout(self, command, record):
-        type = command['type']
-        if type == 'QHBoxLayout': layout = QHBoxLayout()
-        elif type == 'QGridLayout': layout = QGridLayout()
-        elif type == 'QStackedLayout': layout = QStackedLayout()
+        layoutType = command['type']
+        if layoutType == 'QHBoxLayout': layout = QHBoxLayout()
+        elif layoutType == 'QGridLayout': layout = QGridLayout()
+        elif layoutType == 'QStackedLayout': layout = QStackedLayout()
         else: layout = QVBoxLayout()
         layout.setContentsMargins(5,0,5,0)
         record['widget'] = layout
         return self.nextPC()
     
     def r_createGroupBox(self, command, record):
-        groupbox = QGroupBox(self.getRuntimeValue(command['title']))
-        groupbox.setAlignment(Qt.AlignLeft)
-        record['widget'] = groupbox
+        group = QGroupBox(self.getRuntimeValue(command['title']))
+        group.setAlignment(Qt.AlignLeft)
+        record['widget'] = group
         return self.nextPC()
     
     def r_createLabel(self, command, record):
         label = QLabel(str(self.getRuntimeValue(command['text'])))
+        label.setStyleSheet("""
+            background-color: transparent;
+            border: none;
+        """)
         if 'size' in command:
             fm = label.fontMetrics()
             c = label.contentsMargins()
@@ -500,18 +563,36 @@ class Graphics(Handler):
         if 'size' in command:
             fm = pushbutton.fontMetrics()
             c = pushbutton.contentsMargins()
-            w = fm.horizontalAdvance('m') * self.getRuntimeValue(command['size']) +c.left()+c.right()
+            w = fm.horizontalAdvance('m') * self.getRuntimeValue(command['size']) + c.left()+c.right()
             pushbutton.setMaximumWidth(w)
         record['widget'] = pushbutton
         return self.nextPC()
     
     def r_createCheckBox(self, command, record):
         checkbox = QCheckBox(self.getRuntimeValue(command['text']))
+        checkbox.setStyleSheet("""
+            QCheckBox::indicator {
+                border: 1px solid black;
+                border-radius: 3px;
+                background: white;
+                width: 16px;
+                height: 16px;
+            }
+            QCheckBox::indicator:checked {
+                background: #0078d7;
+            }
+            QCheckBox {
+                border: none;
+                background: transparent;
+            }
+        """)
+        checkbox.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Preferred)
         record['widget'] = checkbox
         return self.nextPC()
     
     def r_createLineEdit(self, command, record):
-        lineinput = QLineEdit()
+        lineinput = self.ClickableLineEdit()
+        lineinput.setText(self.getRuntimeValue(command['text']))
         fm = lineinput.fontMetrics()
         m = lineinput.textMargins()
         c = lineinput.contentsMargins()
@@ -521,7 +602,7 @@ class Graphics(Handler):
         return self.nextPC()
     
     def r_createMultiLineEdit(self, command, record):
-        textinput = QPlainTextEdit()
+        textinput = self.ClickablePlainTextEdit()
         fontMetrics = textinput.fontMetrics()
         charWidth = fontMetrics.horizontalAdvance('x')
         charHeight = fontMetrics.height()
@@ -552,13 +633,13 @@ class Graphics(Handler):
             mainLayout.addWidget(QLabel(prompt))
         elif dialogType == 'lineedit':
             mainLayout.addWidget(QLabel(prompt))
-            dialog.lineEdit = QLineEdit(dialog)
+            dialog.lineEdit = self.ClickableLineEdit(dialog)
             dialog.value = self.getRuntimeValue(command['value'])
             dialog.lineEdit.setText(dialog.value)
             mainLayout.addWidget(dialog.lineEdit)
         elif dialogType == 'multiline':
             mainLayout.addWidget(QLabel(prompt))
-            dialog.textEdit = QPlainTextEdit(self)
+            dialog.textEdit = self.ClickablePlainTextEdit(self)
             dialog.textEdit.setText(dialog.value)
             mainLayout.addWidget(dialog.textEdit)
         buttonBox = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
@@ -583,7 +664,7 @@ class Graphics(Handler):
         keyword = record['keyword']
         if keyword == 'window': return self.r_createWindow(command, record)
         elif keyword == 'layout': return self.r_createLayout(command, record)
-        elif keyword == 'groupbox': return self.r_createGroupBox(command, record)
+        elif keyword == 'group': return self.r_createGroupBox(command, record)
         elif keyword == 'label': return self.r_createLabel(command, record)
         elif keyword == 'pushbutton': return self.r_createPushbutton(command, record)
         elif keyword == 'checkbox': return self.r_createCheckBox(command, record)
@@ -627,10 +708,10 @@ class Graphics(Handler):
         return self.nextPC()
 
     # Create a group box
-    def k_groupbox(self, command):
+    def k_group(self, command):
         return self.compileVariable(command, 'gui')
 
-    def r_groupbox(self, command):
+    def r_group(self, command):
         return self.nextPC()
 
     # Initialize the graphics environment
@@ -690,7 +771,7 @@ class Graphics(Handler):
     def r_multiline(self, command):
         return self.nextPC()
 
-    # on click {pushbutton}
+    # on click {pushbutton}/{lineinput}/{multiline}
     # on select {combobox}/{listbox}
     # on tick
     def k_on(self, command):
@@ -723,7 +804,7 @@ class Graphics(Handler):
         if token == 'click':
             if self.nextIsSymbol():
                 record = self.getSymbolRecord()
-                if record['keyword'] == 'pushbutton':
+                if record['keyword'] in ['pushbutton', 'lineinput', 'multiline']:
                     command['name'] = record['name']
                     setupOn()
                     return True
@@ -852,7 +933,7 @@ class Graphics(Handler):
     # set [the] spacing of {layout} to {value}
     # set [the] text [of] {label}/{button}/{lineinput}/{multiline} [to] {text}
     # set [the] color [of] {label}/{button}/{lineinput}/{multiline} [to] {color}
-    # set [the] state [of] {checkbox} [to] {color}
+    # set [the] state [of] {checkbox} [to] {state}
     # set {listbox} to {list}
     # set blocked true/false
     def k_set(self, command):
@@ -895,7 +976,7 @@ class Graphics(Handler):
             self.skip('of')
             if self.nextIsSymbol():
                 record = self.getSymbolRecord()
-                if record['keyword'] in ['label', 'pushbutton', 'lineinput', 'multiline']:
+                if record['keyword'] in ['label', 'pushbutton', 'lineinput', 'multiline', 'element']:
                     command['name'] = record['name']
                     self.skip('to')
                     command['value'] = self.nextValue()
@@ -906,6 +987,29 @@ class Graphics(Handler):
             if self.nextIsSymbol():
                 record = self.getSymbolRecord()
                 if record['keyword'] == 'checkbox':
+                    command['name'] = record['name']
+                    self.skip('to')
+                    command['value'] = self.nextToken()
+                    self.add(command)
+                    return True
+        elif token == 'alignment':
+            self.skip('of')
+            if self.nextIsSymbol():
+                record = self.getSymbolRecord()
+                if record['extra'] == 'gui':
+                    command['name'] = record['name']
+                    self.skip('to')
+                    flags = []
+                    while self.peek() in ['left', 'hcenter', 'right', 'top', 'vcenter', 'bottom', 'center']:
+                        flags.append(self.nextToken())
+                    command['value'] = flags
+                    self.add(command)
+                    return True
+        elif token == 'style':
+            self.skip('of')
+            if self.nextIsSymbol():
+                record = self.getSymbolRecord()
+                if record['keyword'] == 'label':
                     command['name'] = record['name']
                     self.skip('to')
                     command['value'] = self.nextValue()
@@ -967,16 +1071,36 @@ class Graphics(Handler):
             record = self.getVariable(command['name'])
             widget = self.getVariable(command['name'])['widget']
             text = self.getRuntimeValue(command['value'])
-            if isinstance(widget, QLineEdit):
+            keyword = record['keyword']
+            setText = getattr(widget, "setText", None)
+            if callable(setText):
                 widget.setText(text)
-            elif isinstance(widget, QPlainTextEdit):
+            elif keyword == 'multiline':
                 widget.setPlainText(text)
             if record['keyword'] == 'pushbutton':
                 widget.setAccessibleName(text)
         elif what == 'state':
+            record = self.getVariable(command['name'])
+            if record['keyword'] == 'checkbox':
+                state = True if command['value'] == 'checked' else False
+                record['widget'].setChecked(state)
+        elif what == 'alignment':
             widget = self.getVariable(command['name'])['widget']
-            state = self.getRuntimeValue(command['value'])
-            widget.setChecked(state)
+            flags = command['value']
+            alignment = 0
+            for flag in flags:
+                if flag == 'left': alignment |= Qt.AlignLeft
+                elif flag == 'hcenter': alignment |= Qt.AlignHCenter
+                elif flag == 'right': alignment |= Qt.AlignRight
+                elif flag == 'top': alignment |= Qt.AlignTop
+                elif flag == 'vcenter': alignment |= Qt.AlignVCenter
+                elif flag == 'bottom': alignment |= Qt.AlignBottom
+                elif flag == 'center': alignment |= Qt.AlignCenter
+            widget.setAlignment(alignment)
+        elif what == 'style':
+            widget = self.getVariable(command['name'])['widget']
+            styles = self.getRuntimeValue(command['value'])
+            widget.setStyleSheet(styles)
         elif what == 'color':
             widget = self.getVariable(command['name'])['widget']
             color = self.getRuntimeValue(command['value'])
